@@ -1,6 +1,16 @@
 import torch
 
 from ._base import Feature
+from .functional import barrier
+from .functional import empty
+from .functional import expiry_time
+from .functional import log_moneyness
+from .functional import max_log_moneyness
+from .functional import max_moneyness
+from .functional import moneyness
+from .functional import prev_hedge
+from .functional import volatility
+from .functional import zeros
 
 
 class Moneyness(Feature):
@@ -12,22 +22,16 @@ class Moneyness(Feature):
 
     def __init__(self, log=False):
         super().__init__()
-
         self.log = log
 
     def __str__(self):
-        if self.log:
-            return "log_moneyness"
-        else:
-            return "moneyness"
+        return "log_moneyness" if self.log else "moneyness"
 
     def __getitem__(self, i):
-        # spot price: shape (N, 1)
-        s = self.derivative.underlier.prices[..., i].reshape(-1, 1)
-        output = s / self.derivative.strike
         if self.log:
-            output = torch.log(output)
-        return output
+            return log_moneyness(i, derivative=self.derivative)
+        else:
+            return moneyness(i, derivative=self.derivative)
 
 
 class LogMoneyness(Moneyness):
@@ -44,8 +48,7 @@ class ExpiryTime(Feature):
         return "expiry_time"
 
     def __getitem__(self, i):
-        value = self.derivative.maturity - i * self.derivative.underlier.dt
-        return torch.full_like(self.derivative.underlier.prices[:, :1], value)
+        return expiry_time(i, derivative=self.derivative)
 
 
 class Volatility(Feature):
@@ -55,8 +58,7 @@ class Volatility(Feature):
         return "volatility"
 
     def __getitem__(self, i):
-        value = self.derivative.underlier.volatility
-        return torch.full_like(self.derivative.underlier.prices[:, :1], value)
+        return volatility(i, derivative=self.derivative)
 
 
 class PrevHedge(Feature):
@@ -66,11 +68,7 @@ class PrevHedge(Feature):
         return "prev_hedge"
 
     def __getitem__(self, i):
-        if hasattr(self.hedger, "prev_output"):
-            return self.hedger.prev_output.reshape(-1, 1)
-        else:
-            # prices: shape (N, T)
-            return torch.zeros_like(self.derivative.underlier.prices[:, :1])
+        return prev_hedge(i, derivative=self.derivative, hedger=self.hedger)
 
 
 class Barrier(Feature):
@@ -79,44 +77,44 @@ class Barrier(Feature):
     and `0` otherwise.
 
     Args:
-        barrier (float): The price level of the barrier.
+        threshold (float): The price level of the barrier.
         up (bool, default True): If `True`, signifies whether the price has exceeded
             the barrier upward.
             If `False`, signifies whether the price has exceeded the barrier downward.
     """
 
-    def __init__(self, barrier, up=True):
+    def __init__(self, threshold, up=True):
         super().__init__()
-
-        self.barrier = barrier
+        self.threshold = threshold
         self.up = up
 
     def __repr__(self):
-        return self.__class__.__name__ + f"({self.barrier}, up={self.up})"
+        return self.__class__.__name__ + f"({self.threshold}, up={self.up})"
 
     def __getitem__(self, i):
-        if self.up:
-            # shape: (N, i)
-            touch_barrier = (
-                self.derivative.underlier.prices[..., : i + 1] >= self.barrier
-            )
-        else:
-            touch_barrier = (
-                self.derivative.underlier.prices[..., : i + 1] <= self.barrier
-            )
-        return touch_barrier.any(dim=-1, keepdim=True).to(
-            self.derivative.underlier.prices
+        return barrier(
+            i, derivative=self.derivative, threshold=self.threshold, up=self.up
         )
 
 
-class Zero(Feature):
+class Zeros(Feature):
     """A feature of which value is always zero."""
 
     def __str__(self):
-        return "zero"
+        return "zeros"
 
     def __getitem__(self, i):
-        return torch.zeros_like(self.derivative.underlier.prices[:, :1])
+        return zeros(i, derivative=self.derivative)
+
+
+class Empty(Feature):
+    """A feature of which value is always empty."""
+
+    def __str__(self):
+        return "empty"
+
+    def __getitem__(self, i):
+        return empty(i, derivative=self.derivative)
 
 
 class MaxMoneyness(Feature):
@@ -127,30 +125,17 @@ class MaxMoneyness(Feature):
     """
 
     def __init__(self, log=False):
+        super().__init__()
         self.log = log
-        self.moneyness = Moneyness(log=log)
 
     def __str__(self):
-        if self.log:
-            return "max_log_moneyness"
-        else:
-            return "max_moneyness"
+        return "max_log_moneyness" if self.log else "max_moneyness"
 
     def __getitem__(self, i):
-        s = (
-            self.derivative.underlier.prices[..., : i + 1]
-            .max(dim=1, keepdim=True)
-            .values
-        )
-        output = s / self.derivative.strike
         if self.log:
-            output = torch.log(output)
-        return output
-
-    def of(self, derivative=None, hedger=None):
-        super().of(derivative)
-        self.moneyness.of(derivative=derivative, hedger=hedger)
-        return self
+            return max_log_moneyness(i, derivative=self.derivative)
+        else:
+            return max_moneyness(i, derivative=self.derivative)
 
 
 class MaxLogMoneyness(MaxMoneyness):
@@ -177,27 +162,28 @@ class ModuleOutput(Feature, torch.nn.Module):
         >>> from torch.nn import Linear
         >>> from pfhedge.instruments import BrownianStock
         >>> from pfhedge.instruments import EuropeanOption
-        >>> deriv = EuropeanOption(BrownianStock())
-        >>> deriv.simulate(n_paths=3)
+        >>> derivative = EuropeanOption(BrownianStock())
+        >>> derivative.simulate(n_paths=3)
         >>> m = Linear(2, 1)
-        >>> f = ModuleOutput(m, [Moneyness(), ExpiryTime()]).of(deriv)
+        >>> f = ModuleOutput(m, [Moneyness(), ExpiryTime()]).of(derivative)
         >>> f[0]
         tensor([[...],
                 [...],
                 [...]], grad_fn=<AddmmBackward>)
         >>> f
         ModuleOutput(
-          inputs=['moneyness', 'expiry_time'],
+          inputs=['moneyness', 'expiry_time']
           (module): Linear(in_features=2, out_features=1, bias=True)
         )
 
         >>> from pfhedge.nn import BlackScholes
 
         >>> _ = torch.manual_seed(42)
-        >>> deriv = EuropeanOption(BrownianStock())
-        >>> deriv.simulate(n_paths=3)
-        >>> m = BlackScholes(deriv)
-        >>> f = ModuleOutput(m, [LogMoneyness(), ExpiryTime(), Volatility()]).of(deriv)
+        >>> derivative = EuropeanOption(BrownianStock())
+        >>> derivative.simulate(n_paths=3)
+        >>> m = BlackScholes(derivative)
+        >>> f = ModuleOutput(m, [LogMoneyness(), ExpiryTime(), Volatility()])
+        >>> f = f.of(derivative)
         >>> f[0]
         tensor([[...],
                 [...],
@@ -211,7 +197,7 @@ class ModuleOutput(Feature, torch.nn.Module):
         self.inputs = inputs
 
     def extra_repr(self):
-        return f"inputs={[str(f) for f in self.inputs]},"
+        return "inputs=" + str([str(f) for f in self.inputs])
 
     def forward(self, input):
         return self.module(input)
